@@ -126,6 +126,7 @@ class CoreGrilo(GObject.GObject):
         new_state = self._tracker_wrapper.props.tracker_available
         if (source.props.source_id == "grl-tracker-source"
                 and source.props.source_id not in self._wrappers.keys()
+                and TrackerWrapper.location_filter() is not None
                 and new_state == TrackerState.AVAILABLE):
             new_wrapper = GrlTrackerWrapper(
                 source, self._coremodel, self._application, self,
@@ -139,7 +140,8 @@ class CoreGrilo(GObject.GObject):
         elif (source.props.source_id not in self._search_wrappers.keys()
                 and source.props.source_id not in self._wrappers.keys()
                 and source.get_supported_media() & Grl.MediaType.AUDIO
-                and source.supported_operations() & Grl.SupportedOps.SEARCH):
+                and source.supported_operations() & Grl.SupportedOps.SEARCH
+                and "net:internet" not in source.props.source_tags):
             self._search_wrappers[source.props.source_id] = GrlSearchWrapper(
                 source, self._coremodel, self._coreselection, self)
             print("search source", source)
