@@ -31,6 +31,7 @@ gi.require_versions({"Grl": "0.3"})
 from gi.repository import Gio, Grl, GLib, GObject
 
 from gnomemusic.coresong import CoreSong
+from gnomemusic.trackerwrapper import TrackerWrapper
 import gnomemusic.utils as utils
 
 
@@ -58,28 +59,32 @@ class GrlTrackerPlaylists(GObject.GObject):
         return "<GrlTrackerPlaylists>"
 
     def __init__(
-            self, source, coremodel, coreselection, grilo, tracker_wrapper):
+            self, source, coremodel, application, grilo, tracker_wrapper):
         """Initialize GrlTrackerPlaylists.
 
         :param Grl.TrackerSource source: The Tracker source to wrap
         :param CoreModel coremodel: CoreModel instance to use models
                                     from
-        :param CoreSelection coreselection: CoreSelection instance to
-                                            use
+        :param Application application: Application instance
         :param CoreGrilo grilo: The CoreGrilo instance
         :param TrackerWrapper tracker_wrapper: The TrackerWrapper
                                                instance
         """
         super().__init__()
 
+        self._application = application
         self._coremodel = coremodel
-        self._coreselection = coreselection
+        self._coreselection = application.props.coreselection
         self._grilo = grilo
         self._source = source
         self._model = self._coremodel.props.playlists
         self._model_filter = self._coremodel.props.playlists_filter
+        self._user_model_filter = self._coremodel.props.user_playlists_filter
         self._pls_todelete = []
         self._tracker = tracker_wrapper.props.tracker
+        self._window = application.props.window
+
+        self._user_model_filter.set_filter_func(self._user_playlists_filter)
 
         self._fast_options = Grl.OperationOptions()
         self._fast_options.set_resolution_flags(
@@ -90,7 +95,7 @@ class GrlTrackerPlaylists(GObject.GObject):
     def _initial_playlists_fill(self):
         args = {
             "source": self._source,
-            "coreselection": self._coreselection,
+            "application": self._application,
             "grilo": self._grilo,
             "tracker": self._tracker
         }
@@ -106,6 +111,7 @@ class GrlTrackerPlaylists(GObject.GObject):
         for playlist in smart_playlists.values():
             self._model.append(playlist)
 
+        self._window.notifications_popup.push_loading()
         query = """
         SELECT DISTINCT
             rdf:type(?playlist)
@@ -136,14 +142,16 @@ class GrlTrackerPlaylists(GObject.GObject):
             self, source, op_id, media, remaining, data=None, error=None):
         if error:
             print("ERROR", error)
+            self._window.notifications_popup.pop_loading()
             return
         if not media:
             self._coremodel.emit("playlists-loaded")
+            self._window.notifications_popup.pop_loading()
             return
 
         playlist = Playlist(
             media=media, source=self._source, coremodel=self._coremodel,
-            coreselection=self._coreselection, grilo=self._grilo,
+            application=self._application, grilo=self._grilo,
             tracker=self._tracker)
 
         self._model.append(playlist)
@@ -154,6 +162,10 @@ class GrlTrackerPlaylists(GObject.GObject):
     def _playlists_filter(self, playlist):
         return playlist not in self._pls_todelete
 
+    def _user_playlists_filter(self, playlist):
+        return (playlist not in self._pls_todelete
+                and playlist.props.is_smart is False)
+
     def stage_playlist_deletion(self, playlist):
         """Adds playlist to the list of playlists to delete
 
@@ -171,6 +183,8 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._pls_todelete.remove(playlist)
         if deleted is False:
             self._model_filter.set_filter_func(self._playlists_filter)
+            self._user_model_filter.set_filter_func(
+                self._user_playlists_filter)
             return
 
         def _delete_cb(conn, res, data):
@@ -182,7 +196,9 @@ class GrlTrackerPlaylists(GObject.GObject):
                     break
 
             self._model_filter.set_filter_func(self._playlists_filter)
+            self._window.notifications_popup.pop_loading()
 
+        self._window.notifications_popup.push_loading()
         query = """
         DELETE {
             ?playlist a rdfs:Resource .
@@ -233,6 +249,7 @@ class GrlTrackerPlaylists(GObject.GObject):
                 query, self.METADATA_KEYS, options, self._add_user_playlist,
                 callback)
 
+        self._window.notifications_popup.push_loading()
         query = """
             INSERT {
                 _:playlist a nmm:Playlist ;
@@ -282,9 +299,21 @@ class Playlist(GObject.GObject):
 
     def __init__(
             self, media=None, query=None, tag_text=None, source=None,
-            coremodel=None, coreselection=None, grilo=None, tracker=None):
-        super().__init__()
+            coremodel=None, application=None, grilo=None, tracker=None):
 
+        super().__init__()
+        """Initialize a playlist
+
+       :param Grl.Media media: A media object
+       :param string query: Tracker query that returns the playlist
+       :param string tag_text: The non translatable unique identifier
+            of the playlist
+       :param Grl.Source source: The Grilo Tracker source object
+       :param CoreModel coremodel: The CoreModel instance
+       :param Application application: The Application instance
+       :param CoreGrilo grilo: The CoreGrilo instance
+       :param TrackerWrapper tracker: The TrackerWrapper instance
+        """
         if media:
             self.props.pl_id = media.get_id()
             self.props.title = utils.get_media_title(media)
@@ -296,9 +325,10 @@ class Playlist(GObject.GObject):
         self._model = None
         self._source = source
         self._coremodel = coremodel
-        self._coreselection = coreselection
+        self._coreselection = application.props.coreselection
         self._grilo = grilo
         self._tracker = tracker
+        self._window = application.props.window
 
         self._fast_options = Grl.OperationOptions()
         self._fast_options.set_resolution_flags(
@@ -320,6 +350,8 @@ class Playlist(GObject.GObject):
         self._model = value
 
     def _populate_model(self):
+        self._window.notifications_popup.push_loading()
+
         query = """
         SELECT
             rdf:type(?song)
@@ -364,6 +396,7 @@ class Playlist(GObject.GObject):
             if not media:
                 self.props.count = self._model.get_n_items()
                 self.emit("playlist-loaded")
+                self._window.notifications_popup.pop_loading()
                 return
 
             coresong = CoreSong(media, self._coreselection, self._grilo)
@@ -382,11 +415,14 @@ class Playlist(GObject.GObject):
 
         :param str new_name: new playlist name
         """
+        self._window.notifications_popup.push_loading()
+
         def update_cb(conn, res, data):
             # FIXME: Check for failure.
             conn.update_finish(res)
             # FIXME: Requery instead?
             self.props.title = new_name
+            self._window.notifications_popup.pop_loading()
 
         query = """
         INSERT OR REPLACE {
@@ -439,7 +475,9 @@ class Playlist(GObject.GObject):
         def update_cb(conn, res, data):
             # FIXME: Check for failure.
             conn.update_finish(res)
+            self._window.notifications_popup.pop_loading()
 
+        self._window.notifications_popup.push_loading()
         query = """
         INSERT OR REPLACE {
             ?entry nfo:listPosition ?position .
@@ -560,7 +598,7 @@ class Playlist(GObject.GObject):
                 )
             }
             """.replace("\n", " ").strip() % {
-                "filter_clause": "tracker:id(?entry) = " + media_id}
+                "filter_clause": "tracker:id(?song) = " + media_id}
             options = self._fast_options.copy()
             self._source.query(
                 query, self.METADATA_KEYS, options, _add_to_model)
@@ -654,13 +692,17 @@ class SmartPlaylist(Playlist):
         if self._model is None:
             self._model = Gio.ListStore.new(CoreSong)
 
+            self._window.notifications_popup.push_loading()
+
             def _add_to_model(source, op_id, media, remaining, error):
                 if error:
                     print("ERROR", error)
+                    self._window.notifications_popup.pop_loading()
                     return
 
                 if not media:
                     self.props.count = self._model.get_n_items()
+                    self._window.notifications_popup.pop_loading()
                     return
 
                 coresong = CoreSong(media, self._coreselection, self._grilo)
@@ -703,9 +745,12 @@ class MostPlayed(SmartPlaylist):
                     nie:usageCounter ?count .
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
+            %(location_filter)s
         }
         ORDER BY DESC(?count) LIMIT 50
-        """.replace('\n', ' ').strip()
+        """.replace('\n', ' ').strip() % {
+            "location_filter": TrackerWrapper.location_filter()
+        }
 
 
 class NeverPlayed(SmartPlaylist):
@@ -737,8 +782,11 @@ class NeverPlayed(SmartPlaylist):
             FILTER ( NOT EXISTS { ?song nie:usageCounter ?count .} )
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
+            %(location_filter)s
         } ORDER BY nfo:fileLastAccessed(?song) LIMIT 50
-        """.replace('\n', ' ').strip()
+        """.replace('\n', ' ').strip() % {
+            "location_filter": TrackerWrapper.location_filter()
+        }
 
 
 class RecentlyPlayed(SmartPlaylist):
@@ -779,9 +827,11 @@ class RecentlyPlayed(SmartPlaylist):
                      && EXISTS { ?song nie:usageCounter ?count .} )
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
+            %(location_filter)s
         } ORDER BY DESC(?last_played) LIMIT 50
         """.replace('\n', ' ').strip() % {
-            'compare_date': compare_date
+            'compare_date': compare_date,
+            "location_filter": TrackerWrapper.location_filter()
         }
 
 
@@ -822,9 +872,11 @@ class RecentlyAdded(SmartPlaylist):
             FILTER ( tracker:added(?song) > '%(compare_date)s'^^xsd:dateTime )
             OPTIONAL { ?song nao:hasTag ?tag .
                        FILTER (?tag = nao:predefined-tag-favorite) }
+            %(location_filter)s
         } ORDER BY DESC(tracker:added(?song)) LIMIT 50
         """.replace('\n', ' ').strip() % {
             'compare_date': compare_date,
+            "location_filter": TrackerWrapper.location_filter()
         }
 
 
@@ -857,8 +909,10 @@ class Favorites(SmartPlaylist):
                         nie:isStoredAs ?as ;
                         nao:hasTag nao:predefined-tag-favorite .
                 ?as nie:url ?url .
-            OPTIONAL { ?song nao:hasTag ?tag .
-                       FILTER (?tag = nao:predefined-tag-favorite) }
-
+                OPTIONAL { ?song nao:hasTag ?tag .
+                           FILTER (?tag = nao:predefined-tag-favorite) }
+                %(location_filter)s
             } ORDER BY DESC(tracker:added(?song))
-        """.replace('\n', ' ').strip()
+        """.replace('\n', ' ').strip() % {
+            "location_filter": TrackerWrapper.location_filter()
+        }
