@@ -55,6 +55,7 @@ class Window(Gtk.ApplicationWindow):
 
     __gtype_name__ = "Window"
 
+    active_view = GObject.Property(type=GObject.GObject, default=None)
     selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
@@ -166,6 +167,8 @@ class Window(Gtk.ApplicationWindow):
         # if some music is available, populate stack with mainviews,
         # show stack and set empty_view to empty_search_view
         self.views[View.EMPTY] = EmptyView()
+        self._stack.connect(
+            "notify::visible-child", self._on_stack_visible_child_changed)
         self._stack.add_named(self.views[View.EMPTY], "emptyview")
 
         # Add the 'background' styleclass so it properly hides the
@@ -224,6 +227,9 @@ class Window(Gtk.ApplicationWindow):
         else:
             self._switch_to_empty_view()
 
+    def _on_stack_visible_child_changed(self, klass, value):
+        self.props.active_view = self._stack.props.visible_child
+
     def _on_tracker_available(self, klass, value):
         grilo = self._app.props.coremodel.props.grilo
         new_state = grilo.props.tracker_available
@@ -300,9 +306,9 @@ class Window(Gtk.ApplicationWindow):
         if not self.props.selection_mode:
             return
         if self._headerbar.props.state == HeaderBar.State.MAIN:
-            view = self._stack.get_visible_child()
+            view = self.props.active_view
         else:
-            view = self._stack.get_visible_child().get_visible_child()
+            view = self.props.active_view.get_visible_child()
 
         view.select_all()
 
@@ -311,10 +317,10 @@ class Window(Gtk.ApplicationWindow):
         if not self.props.selection_mode:
             return
         if self._headerbar.props.state == HeaderBar.State.MAIN:
-            view = self._stack.get_visible_child()
+            view = self.props.active_view
             view.unselect_all()
         else:
-            view = self._stack.get_visible_child().get_visible_child()
+            view = self.props.active_view.get_visible_child()
             view.select_none()
 
     @log
@@ -399,7 +405,7 @@ class Window(Gtk.ApplicationWindow):
             if keyval == Gdk.KEY_AudioNext:
                 self._player.next()
 
-            child = self._stack.get_visible_child()
+            child = self.props.active_view
             if (keyval == Gdk.KEY_Delete
                     and child == self.views[View.PLAYLIST]):
                 self.views[View.PLAYLIST].remove_playlist()
@@ -434,7 +440,7 @@ class Window(Gtk.ApplicationWindow):
     @log
     def _on_notify_mode(self, stack, param):
         self.prev_view = self.curr_view
-        self.curr_view = stack.get_visible_child()
+        self.curr_view = self.props.active_view
 
         # Disable search mode when switching view
         search_views = [self.views[View.EMPTY], self.views[View.SEARCH]]
@@ -500,7 +506,7 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _on_add_to_playlist(self, widget):
-        if self._stack.get_visible_child() == self.views[View.PLAYLIST]:
+        if self.props.active_view == self.views[View.PLAYLIST]:
             return
 
         selected_songs = self._app._coreselection.props.selected_items
